/*
   Example plugin showing how to hook into imap client's destroy() method:

   eval `cat /usr/local/lib/dovecot/dovecot-config`
   gcc -fPIC -shared -DHAVE_CONFIG_H \
     `echo $DOVECOT_CFLAGS $LIBDOVECOT_INCLUDE $LIBDOVECOT_STORAGE_INCLUDE` \
     imap-logout-plugin.c -o imap_logout_plugin.so
*/

#include "imap-common.h"
#include "module-context.h"
#include "imap-client.h"
#include "mail-user.h"

#define IMAP_LOGOUT_IMAP_CONTEXT(obj) \
	MODULE_CONTEXT(obj, imap_logout_imap_module)

struct logout_client {
	union imap_module_context module_ctx;
};

void imap_logout_plugin_init(struct module *module);
void imap_logout_plugin_deinit(void);

static imap_client_created_func_t *next_hook_client_created;

static MODULE_CONTEXT_DEFINE_INIT(imap_logout_imap_module,
				  &imap_module_register);

static void
imap_logout_client_destroy(struct client *client, const char *reason)
{
	struct logout_client *lclient = IMAP_LOGOUT_IMAP_CONTEXT(client);

	i_info("%s logs out", client->user->username);

	lclient->module_ctx.super.destroy(client, reason);
}

static void imap_logout_client_created(struct client **clientp)
{
	struct client *client = *clientp;
	struct logout_client *lclient;

	lclient = p_new(client->pool, struct logout_client, 1);
	lclient->module_ctx.super = client->v;
	client->v.destroy = imap_logout_client_destroy;
	MODULE_CONTEXT_SET(client, imap_logout_imap_module, lclient);

	if (next_hook_client_created != NULL)
		next_hook_client_created(clientp);
}

void imap_logout_plugin_init(struct module *module ATTR_UNUSED)
{
	next_hook_client_created =
		imap_client_created_hook_set(imap_logout_client_created);
}

void imap_logout_plugin_deinit(void)
{
	imap_client_created_hook_set(next_hook_client_created);
}
